#! /usr/bin/env python

from __future__ import print_function

"""---------------------------------------------------------------------------
aosong_am2315.py

Written by Sopwith
03 May 2014

Redistribution and use in source and binary forms, with or without
modification, are permitted. Use of this software for illegal or
malicious purposed is strictly prohibited. Attribution to the creator(s)
is appreciated but not required.

THIS SOFTWARE IS PROVIDED BY THE CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#-----------------------------------------------------------------------------
 Python application that wraps the capabilities of the AOSONG AM2315 humidity
 and temperature sensor.
 The datasheet for the device can be found here:
 http://www.adafruit.com/datasheets/AM2315.pdf
#-----------------------------------------------------------------------------

#-----------------------------------------------------------------------------
 NOTE: Special thanks to lexruee for his great code library.
#-----------------------------------------------------------------------------

#-----------------------------------------------------------------------------
 Revision history:
     2017-11-12 05:45 UTC-8 - Sopwith - Modified to use tentacle_pi library.	
     2016-04-12 15:03 UTC-7 - Sopwith - Fixed negative temperature calcs.
     2016-04-12 15:03 UTC-7 - Sopwith - Added calc_dewpoint().
     2014-05-01 - Initial release
#--------------------------------------------------------------------------"""

from tentacle_pi.AM2315 import AM2315
import time
import math

am = None

def main():

    global am
    am = AM2315(0x5c, "/dev/i2c-1")
    sensor = AOSONG_AM2315()
    
    data = sensor.ReadSensorData()
    print('Humidity = ', data[0], '%', sep='')
    print('Temperature = ', data[1], 'C', sep='')
    print('Temperature = ', data[2], 'F', sep='')

    dp = sensor.calc_dewpoint(data[1], data[0])
    print('Dewpoint = ', dp[0], 'C ', dp[1], 'F', sep='')	


class AOSONG_AM2315:
    """Wrapping for an AOSONG AM2315 humidity and temperature sensor.

    Provides simple access to a AM2315 chip.
    Attributes:
        lastError: String containing the last error string.Formatter
        debug:     bool containing debug state
    """

    def __init__(self, address=0x5C, debug=False):
        self.lastError = None  # Contains last error string
        self.debug = debug  # Debug flag


    def ReadSensorData(self):
        """ Reads the humidity and temperature from the AS2315.

        Args:
            None
        Returns:
            Tuple containing the following fields:
                humidity    - float
                temperature - float (Celsius)
                temperature - float (Fahrenheit)
        """
	global am
        data = None

	tempC, humidity, crc_check = am.sense()
	tempC = round(tempC, 1)
	humidity = round(humidity,2)
	tempF = self.CelsiusToFahrenheit(tempC)

        return (humidity, tempC, tempF)


    def ReadHumidity(self):
        """Read humidity data from the sensor.

        Args:
            None
        Returns:
            float = humidity reading, None if error
        """
        time.sleep(.25)
        data = self.ReadSensorData()
        if data != None:
            return self.ReadSensorData()[0]
        return None


    def ReadTemperature(self, fahrenheit=False):
        """Read temperature data from the sensor. (Celsius is default)

        Args:
            bool - if True returns temp in Fahrenheit. Default=False
        Returns:
            float = humidity reading, None if error
        """
        time.sleep(.25)
        data = self.ReadSensorData()
        if data == None:
            return None
        if fahrenheit:
            return self.ReadSensorData()[2]
        return self.ReadSensorData()[1]


    def CelsiusToFahrenheit(self, celsius):
        """Convert Celsius to Fahrenheit.

        Params:
            celsius: int containing C temperature

        Returns:
            String with Fahrenheit conversion. None if error.
        """
        if celsius == None:
            return

        if celsius == 0:
            return 32

        try:
            tempF = float((celsius * 9 / 5) + 32)
            return (round(tempF,1))
        except:

            self.lastError = 'Error converting %s celsius to fahrenheit' % celsius
            return None


    def calc_dewpoint(self, temp, rh):
        """Calculate dewpoint from temp and humidity.

        Calculation used in this method was obtained here:
        http://andrew.rsmas.miami.edu/bmcnoldy/Humidity.html

        Algorithm used:
        243.04*(LN(RH/100)+((17.625*T)/(243.04+T)))/(17.625-LN(RH/100)-
        ((17.625*T)/(243.04+T)))

        Params:
            temp: int or float containing C temperature

        Returns:
            Tuple containing C and F dewpoint. None if error.
        """
        assert temp
        assert rh

        dp_C = 243.04 * (
        math.log(rh * .01) + ((17.625 * temp) / (243.04 + temp))) / (
             17.625 - math.log(rh * .01) - ((17.625 * temp) / (243.04 + temp)))

        dp_C = math.trunc(dp_C*100) / 100
        dp_F = self.CelsiusToFahrenheit(dp_C)
        dp_F = math.trunc(dp_F*100) / 100

        return (dp_C, dp_F)


    def GetLastError(self):
        return self.lastError


if __name__ == "__main__":
    main()
